<?php

/**
 * @file
 * WA product component.
 *
 * Used for appointment product selection.
 */

/**
 * Implements _webform_defaults_COMPONENT().
 */
function _webform_defaults_wa_product() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'required' => 0,
    'pid' => 0,
    'weight' => 0,
    'extra' => array(
      'options' => '',
      'questions' => '',
      'optrand' => 0,
      'qrand' => 0,
      'description' => '',
      'private' => FALSE,
      'analysis' => TRUE,
    ),
  );
}

/**
 * Implements _webform_edit_COMPONENT().
 */
function _webform_edit_wa_product($component) {
  $form = array();

  // Disabling the description if not wanted.
  $form['extra']['description'] = array();

  return $form;
}

/**
 * Implements _webform_submit_COMPONENT().
 */
function _webform_submit_wa_product($component, $value) {
  if (!module_exists('wfm') || !_wfm_is_multiple($component)) {
    $value = array($value);
  }

  return $value;
}

/**
 * Implements _webform_display_COMPONENT().
 */
function _webform_display_wa_product($component, $values, $format = 'html') {
  if (isset($values['product'])) {
    $values = array($values);
  }

  $client = dvg_appointments_get_client_api();
  $products = $client->get_filtered_available_products();
  $display = array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#required' => $component['required'],
    '#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#post_render' => array('webform_element_wrapper'),
    '#component' => $component,
  );

  foreach ($values as $value) {
    $product = $products[$value['product']];
    if ($format == 'html') {
      $display_value = theme('table', array(
        'rows' => array(
          array(t('Product'), $product['name']),
          array(t('Number of persons'), $value['count']),
        ),
      ));
    }
    else {
      $display_value = t('Product') . ' = ' . $product['name'] . "\n" . t('Number of persons') . ' = ' . $value['count'];
    }

    $display[] = array(
      '#format' => $format,
      '#markup' => $display_value,
      '#parents' => array(FALSE),
      '#value' => $value,
    );
  }

  return $display;
}

/**
 * Implements _webform_render_COMPONENT().
 */
function _webform_render_wa_product($component, $value = NULL, $filter = TRUE) {
  $form_item = array(
    '#type' => 'wa_product',
    '#title' => $filter ? webform_filter_xss($component['name']) : $component['name'],
    '#required' => $component['required'],
    '#weight' => $component['weight'],
    '#description'   => $filter ? webform_filter_descriptions($component['extra']['description']) : $component['extra']['description'],
    '#default_value' => $filter ? webform_replace_tokens($component['value']) : $component['value'],
    '#prefix' => '<div class="webform-component-textfield" id="webform-component-' . $component['form_key'] . '">',
    '#suffix' => '</div>',
  );

  if (!empty($value)) {
    $form_item['#default_value'] = $value;
  }

  return $form_item;
}

/**
 * Element process callback for the wa_product elements.
 */
function dvg_appointments_product_element_process($element, &$form_state, $form) {
  $client = dvg_appointments_get_client_api();
  $product_options = array('' => t('Select a product'));
  $invisible_values = array(array('value' => ''));

  $available_products = $client->get_filtered_available_products();
  // Let other modules lookup the appointment.
  drupal_alter('dvg_appointments_available_products', $available_products, $element, $form_state, $form);

  foreach ($available_products as $product) {
    $product_options[$product['id']] = $product['name'];

    if (isset($product['max_persons']) && $product['max_persons'] == 1) {
      $invisible_values[] = array('value' => strval($product['id']));
    }
  }

  $element['product'] = array(
    '#type' => 'select',
    '#title' => t('Product'),
    '#required' => TRUE,
    '#options' => $product_options,
    '#attributes' => array('class' => array('wa-product-product')),
  );

  if (!empty($element['#default_value']['product'])) {
    $element['product']['#default_value'] = $element['#default_value']['product'];
  }
  elseif (isset($_GET['product_code']) && isset($product_options[$_GET['product_code']])) {
    $element['product']['#default_value'] = $_GET['product_code'];
  }

  $element['count'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of persons'),
    '#required' => TRUE,
    '#size' => 4,
    '#maxlength' => 4,
    '#default_value' => !empty($element['#default_value']['count']) ? $element['#default_value']['count'] : 1,
    '#element_validate' => array('element_validate_integer_positive'),
    '#attributes' => array('class' => array('wa-product-count')),
  );

  if (count($invisible_values) > 1) {
    $element['count']['#states']['invisible'] = array(
      ':input[name="' . $element['#name'] . '[product]"]' => $invisible_values,
    );

    $element['#attached']['js'][] = array(
      'type' => 'setting',
      'data' => array(
        'dvg_appointments' => array(
          $element['#name'] . '[count]' => TRUE,
        ),
      ),
    );

  }

  $element['#element_validate'][] = '_dvg_appointments_element_validate_person_count';

  return $element;
}

/**
 * Element validate callback for the wa_product elements.
 */
function dvg_appointments_product_element_validate($element, &$form_state) {
  $clash_found = FALSE;
  $products = array();
  $form_key = $element['#webform_component']['form_key'];

  if (module_exists('wfm') && _wfm_is_multiple($element['#webform_component']) && !empty($form_state['values']['submitted'][$form_key])) {
    foreach ($form_state['values']['submitted'][$form_key] as $index => $product) {
      if (isset($product['product']) && !empty($product['product'])) {
        if (isset($products[$product['product']])) {
          $clash_found = TRUE;
        }
        $products[$product['product']] = $product;
      }
    }
    if ($clash_found) {
      form_error($element['product'], t('Product must be unique'));
    }
  }
}

/**
 * Returns HTML for the product/count fields on form.
 */
function theme_dvg_appointments_product($variables) {
  $element = $variables['element'];

  // Group start/end items together in fieldset.
  $fieldset = array(
    '#title' => t($element['#title']) . ' ' . ($element['#delta'] > 0 ? intval($element['#delta'] + 1) : ''),
    '#value' => '',
    '#description' => !empty($element['#fieldset_description']) ? $element['#fieldset_description'] : '',
    '#attributes' => array(),
    '#children' => $element['#children'],
  );

  // Add marker to required date fields.
  if ($element['#required']) {
    $fieldset['#title'] .= ' ' . theme('form_required_marker');
  }

  return theme('fieldset', array('element' => $fieldset));
}
