<?php
function dvg_appointments_admin_form($form, $form_state) {
  $api_clients = array();
  $all_api_clients = module_invoke_all('dvg_appointments_api_client');
  foreach ($all_api_clients as $id => $info) {
    $api_clients[$id] = $info['name'];
  }
  //$current_api_client = (isset($form_state['values']['current_client_api'])) ? $form_state['values']['current_client_api'] : variable_get('dvg_appointments_api_client', FALSE);
  $current_api_client = variable_get('dvg_appointments_api_client', FALSE);

  $form['dvg_appointments_api_client'] = array(
    '#title' => t('Client API'),
    '#type' => 'select',
    '#required' => TRUE,
    '#default_value' => $current_api_client,
    '#options' => $api_clients,
  );
  if ($current_api_client) {
    $class = $all_api_clients[$current_api_client]['class'];

    $form['client_api_options'] = array(
      '#title' => t('@name options', array('@name' => $api_clients[$current_api_client])),
      '#type' => 'fieldset',
    );

    $form['client_api_options'][] = $class::config_form();
  }

  $form['change_form'] = array(
    '#title' => t('Changing appointments'),
    '#type' => 'fieldset',
  );
  $form['change_form']['dvg_appointments_allow_changing_appointments'] = array(
    '#title' => t('Allow changing appointments'),
    '#description' => t('When enabled, users will be provided with a change form that allows rescheduling their appointment.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('dvg_appointments_allow_changing_appointments', FALSE),
  );
  $form['change_form']['dvg_appointments_change_form_urls'] = array(
    '#title' => t('Specify the URLs for the change appointment form per language'),
    '#description' => t('You can only set the text between the domain and "/webform".'),
    '#tree' => TRUE,
  );
  foreach (locale_language_list() as $lang_code => $lang_name) {
    $form['change_form']['dvg_appointments_change_form_urls'][$lang_code] = array(
      '#title' => t('URL for !lang', array('!lang' => $lang_name)),
      '#type' => 'textfield',
      '#default_value' => _dvg_appointments_get_change_form_url_part($lang_code),
      '#states' => array(
        'visible' => array(
          'input[name="dvg_appointments_allow_changing_appointments"]' => array(
            'checked' => TRUE,
          ),
        ),
      ),
    );
  }

  $form['flood_control'] = array(
    '#title' => t('Flood control'),
    '#type' => 'fieldset',
  );
  $form['flood_control']['dvg_appointments_flood_threshold'] = array(
    '#title' => t('Threshold'),
    '#description' => t('Leave empty to disable flood control.'),
    '#type' => 'textfield',
    '#size' => 4,
    '#element_validate' => array('element_validate_integer_positive'),
    '#field_suffix' => t('attempts'),
    '#default_value' => variable_get('dvg_appointments_flood_threshold', ''),
  );
  $form['flood_control']['dvg_appointments_flood_timeout'] = array(
    '#title' => t('Timeout'),
    '#type' => 'textfield',
    '#size' => 4,
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
    '#field_suffix' => t('minutes'),
    '#default_value' => variable_get('dvg_appointments_flood_timeout', 15),
  );

  $form = system_settings_form($form);
  $form['#submit'][] = 'dvg_appointments_admin_form_submit';
  return $form;
}

/**
 * Submit callback for the appointments admin form.
 *
 * Runs after the default system settings submit callback.
 */
function dvg_appointments_admin_form_submit($form, &$form_state) {
  // Clear menu cache to register new change form paths.
  menu_cache_clear_all();
}
